@@ -1,5 +1,3 @@
-use std::sync::Arc;
-
 use datafusion::arrow;
 
 use crate::error::QueryError;
@@ -9,12 +7,12 @@ pub async fn exec_query(
     sql: &str,
 ) -> Result<Vec<arrow::record_batch::RecordBatch>, QueryError> {
     let plan = dfctx
+        .state()
         .create_logical_plan(sql)
+        .await
         .map_err(QueryError::plan_sql)?;
 
-    let df: Arc<datafusion::dataframe::DataFrame> = Arc::new(
-        datafusion::dataframe::DataFrame::new(dfctx.state.clone(), &plan),
-    );
+    let df = datafusion::dataframe::DataFrame::new(dfctx.state(), plan);
 
     df.collect().await.map_err(QueryError::query_exec)
 }
@@ -28,9 +26,9 @@ mod tests {
     use crate::test_util::*;
 
     #[tokio::test]
-    async fn group_by_aggregation() -> anyhow::Result<()> {
+    async fn group_by_aggregation() {
         let mut dfctx = SessionContext::new();
-        register_table_properties(&mut dfctx)?;
+        register_table_properties(&mut dfctx);
 
         let batches = exec_query(
             &dfctx,
@@ -41,23 +39,19 @@ mod tests {
             ORDER BY landlord
             "#,
         )
-        .await?;
+        .await
+        .unwrap();
 
         let batch = &batches[0];
 
         assert_eq!(
             batch.column(0).as_ref(),
-            Arc::new(StringArray::from(vec![
-                "Carl", "Daniel", "Mike", "Roger", "Sam",
-            ]))
-            .as_ref(),
+            &StringArray::from(vec!["Carl", "Daniel", "Mike", "Roger", "Sam",]),
         );
 
         assert_eq!(
             batch.column(1).as_ref(),
-            Arc::new(Int64Array::from(vec![3, 3, 4, 3, 2])).as_ref(),
+            &Int64Array::from(vec![3, 3, 4, 3, 2]),
         );
-
-        Ok(())
     }
 }
