@@ -1,12 +1,14 @@
 use std::ffi::OsStr;
+use std::future::Future;
 use std::io::Read;
 use std::path::Path;
+use std::pin::Pin;
 use std::sync::Arc;
 
-use datafusion::datasource::TableProvider;
-
 use datafusion::arrow;
+use datafusion::datasource::file_format::csv::CsvFormat;
 use datafusion::datasource::listing::{ListingOptions, ListingTableUrl};
+use datafusion::datasource::TableProvider;
 use serde::de::{Deserialize, Deserializer};
 use serde_derive::Deserialize;
 use snafu::prelude::*;
@@ -216,6 +218,18 @@ impl TableOptionCsv {
     pub fn default_use_memory_table() -> bool {
         true
     }
+
+    pub fn as_arrow_csv_format(&self) -> arrow::csv::reader::Format {
+        arrow::csv::reader::Format::default()
+            .with_delimiter(self.delimiter)
+            .with_header(self.has_header)
+    }
+
+    pub fn as_df_csv_format(&self) -> CsvFormat {
+        CsvFormat::default()
+            .with_has_header(self.has_header)
+            .with_delimiter(self.delimiter)
+    }
 }
 
 impl Default for TableOptionCsv {
@@ -309,9 +323,15 @@ pub enum TableLoadOption {
     delta(TableOptionDelta),
     arrow {},
     arrows {},
-    mysql {},
-    sqlite {},
-    postgres {},
+    mysql {
+        table: Option<String>,
+    },
+    sqlite {
+        table: Option<String>,
+    },
+    postgres {
+        table: Option<String>,
+    },
 }
 
 impl TableLoadOption {
@@ -406,18 +426,25 @@ impl std::fmt::Display for TableIoSource {
     }
 }
 
+fn table_name_from_path(path: &uriparse::Path) -> Option<String> {
+    Some(path.segments()[0].to_string())
+}
+
 #[derive(Deserialize, Clone, Debug, Eq, PartialEq)]
 #[serde(deny_unknown_fields)]
 pub struct TableSource {
     pub name: String,
     #[serde(flatten)]
     pub io_source: TableIoSource,
+    pub io_option: Option<io::IoOption>,
     pub schema: Option<TableSchema>,
     pub schema_from_files: Option<Vec<String>>,
     pub option: Option<TableLoadOption>,
     #[serde(default = "TableSource::default_batch_size")]
     pub batch_size: usize,
     pub partition_columns: Option<Vec<TableColumn>>,
+    #[serde(default = "TableSource::default_reload_interval")]
+    pub reload_interval: Option<tokio::time::Duration>,
 }
 
 impl From<KeyValueSource> for TableSource {
@@ -430,6 +457,8 @@ impl From<KeyValueSource> for TableSource {
             option: kv.option,
             batch_size: Self::default_batch_size(),
             partition_columns: None,
+            reload_interval: None,
+            io_option: None,
         }
     }
 }
@@ -446,6 +475,8 @@ impl TableSource {
             option,
             batch_size: Self::default_batch_size(),
             partition_columns: None,
+            reload_interval: Self::default_reload_interval(),
+            io_option: None,
         }
     }
 
@@ -466,6 +497,11 @@ impl TableSource {
         8192
     }
 
+    #[inline]
+    pub fn default_reload_interval() -> Option<std::time::Duration> {
+        None
+    }
+
     #[inline]
     #[must_use]
     pub fn with_option(mut self, option: impl Into<TableLoadOption>) -> Self {
@@ -473,6 +509,12 @@ impl TableSource {
         self
     }
 
+    #[inline]
+    pub fn with_io_option(mut self, io_option: io::IoOption) -> Self {
+        self.io_option = Some(io_option);
+        self
+    }
+
     #[inline]
     #[must_use]
     pub fn with_schema(mut self, schema: impl Into<TableSchema>) -> Self {
@@ -492,6 +534,11 @@ impl TableSource {
         self
     }
 
+    pub fn with_reload_interval(mut self, interval: std::time::Duration) -> Self {
+        self.reload_interval = Some(interval);
+        self
+    }
+
     pub fn get_uri_str(&self) -> &str {
         match &self.io_source {
             TableIoSource::Uri(uri) => uri.as_str(),
@@ -505,9 +552,16 @@ impl TableSource {
                 let uri = URIReference::try_from(uri.as_str()).ok()?;
                 let scheme = uri.scheme()?;
                 match scheme.as_str() {
-                    "mysql" => Some(TableLoadOption::mysql {}),
-                    "sqlite" => Some(TableLoadOption::sqlite {}),
-                    "postgresql" => Some(TableLoadOption::postgres {}),
+                    "mysql" => Some(TableLoadOption::mysql {
+                        table: table_name_from_path(uri.path()),
+                    }),
+                    "sqlite" => Some(TableLoadOption::sqlite {
+                        // for sqlite, db uri only contains the path to the db file
+                        table: None,
+                    }),
+                    "postgresql" => Some(TableLoadOption::postgres {
+                        table: table_name_from_path(uri.path()),
+                    }),
                     _ => None,
                 }
             }
@@ -558,9 +612,9 @@ impl TableSource {
                     None => {
                         // database sources doesn't have suffix extension, parse scheme instead
                         match TableSource::parse_option(&self.io_source) {
-                            Some(TableLoadOption::mysql {}) => "mysql",
-                            Some(TableLoadOption::sqlite {}) => "sqlite",
-                            Some(TableLoadOption::postgres {}) => "postgres",
+                            Some(TableLoadOption::mysql { .. }) => "mysql",
+                            Some(TableLoadOption::sqlite { .. }) => "sqlite",
+                            Some(TableLoadOption::postgres { .. }) => "postgres",
                             _ => {
                                 return Err(Error::Extension {
                                     msg: format!("unsupported extension in uri: {uri}"),
@@ -604,14 +658,13 @@ pub async fn datafusion_get_or_infer_schema(
                         .expect("Failed to create file url"),
                 )
                 .context(InferSchemaSnafu)?;
+                let inferred_schema = listing_options
+                    .infer_schema(&dfctx.state(), &file_url)
+                    .await
+                    .context(InferSchemaSnafu)?;
                 schemas.push(
-                    Arc::into_inner(
-                        listing_options
-                            .infer_schema(&dfctx.state(), &file_url)
-                            .await
-                            .context(InferSchemaSnafu)?,
-                    )
-                    .expect("Failed to unwrap schemaref into schema on merge"),
+                    Arc::into_inner(inferred_schema)
+                        .expect("Failed to unwrap schemaref into schema on merge"),
                 );
             }
             Arc::new(arrow::datatypes::Schema::try_merge(schemas).context(MergeSchemaSnafu)?)
@@ -623,65 +676,105 @@ pub async fn datafusion_get_or_infer_schema(
     })
 }
 
+pub type TableRefresherOutput =
+    Pin<Box<dyn Future<Output = Result<Arc<dyn TableProvider>, Error>> + Send>>;
+
+pub type TableRefresher = Box<dyn FnMut() -> TableRefresherOutput + Send>;
+
+pub struct LoadedTable {
+    pub table: Arc<dyn TableProvider>,
+    pub refresher: TableRefresher,
+}
+
+impl LoadedTable {
+    pub fn new(table: Arc<dyn TableProvider>, refresher: TableRefresher) -> Self {
+        Self { table, refresher }
+    }
+
+    pub async fn new_from_df_table_cb<F, Fut>(to_df_table_cb: F) -> Result<Self, Error>
+    where
+        F: Fn() -> Fut + Send + 'static,
+        Fut: Future<Output = Result<Arc<dyn TableProvider>, Error>> + Send + 'static,
+    {
+        let refresher =
+            Box::new(move || Box::pin(to_df_table_cb()) as crate::table::TableRefresherOutput);
+        Ok(Self::new(refresher().await?, refresher))
+    }
+
+    pub fn new_from_df_table(table: Arc<dyn TableProvider>) -> Self {
+        let t = table.clone();
+        let refresher = Box::new(move || {
+            let t = table.clone();
+            let fut = async { Ok(t) };
+            Box::pin(fut) as crate::table::TableRefresherOutput
+        });
+        Self::new(t, refresher)
+    }
+}
+
 pub async fn load(
     t: &TableSource,
     dfctx: &datafusion::execution::context::SessionContext,
-) -> Result<Arc<dyn TableProvider>, Error> {
+) -> Result<LoadedTable, Error> {
     if let Some(opt) = &t.option {
         Ok(match opt {
-            TableLoadOption::json { .. } => Arc::new(json::to_mem_table(t, dfctx).await?),
+            TableLoadOption::json { .. } => json::to_loaded_table(t.clone(), dfctx.clone()).await?,
             TableLoadOption::ndjson { .. } | TableLoadOption::jsonl { .. } => {
-                Arc::new(ndjson::to_mem_table(t, dfctx).await?)
+                ndjson::to_loaded_table(t.clone(), dfctx.clone()).await?
+            }
+            TableLoadOption::csv { .. } => csv::to_loaded_table(t.clone(), dfctx.clone()).await?,
+            TableLoadOption::parquet { .. } => {
+                parquet::to_loaded_table(t.clone(), dfctx.clone()).await?
             }
-            TableLoadOption::csv { .. } => csv::to_datafusion_table(t, dfctx).await?,
-            TableLoadOption::parquet { .. } => parquet::to_datafusion_table(t, dfctx).await?,
             TableLoadOption::google_spreadsheet(_) => {
-                Arc::new(google_spreadsheets::to_mem_table(t).await?)
+                google_spreadsheets::to_loaded_table(t).await?
             }
             TableLoadOption::xlsx { .. }
             | TableLoadOption::xls { .. }
             | TableLoadOption::xlsb { .. }
-            | TableLoadOption::ods { .. } => Arc::new(excel::to_mem_table(t).await?),
-            TableLoadOption::delta { .. } => delta::to_datafusion_table(t, dfctx).await?,
+            | TableLoadOption::ods { .. } => excel::to_loaded_table(t.clone()).await?,
+            TableLoadOption::delta { .. } => delta::to_loaded_table(t, dfctx).await?,
             TableLoadOption::arrow { .. } => {
-                Arc::new(arrow_ipc_file::to_mem_table(t, dfctx).await?)
+                arrow_ipc_file::to_loaded_table(t.clone(), dfctx.clone()).await?
             }
             TableLoadOption::arrows { .. } => {
-                Arc::new(arrow_ipc_stream::to_mem_table(t, dfctx).await?)
-            }
-            TableLoadOption::mysql { .. } => {
-                Arc::new(database::DatabaseLoader::MySQL.to_mem_table(t)?)
-            }
-            TableLoadOption::sqlite { .. } => {
-                Arc::new(database::DatabaseLoader::SQLite.to_mem_table(t)?)
-            }
-            TableLoadOption::postgres { .. } => {
-                Arc::new(database::DatabaseLoader::Postgres.to_mem_table(t)?)
+                arrow_ipc_stream::to_loaded_table(t.clone(), dfctx.clone()).await?
             }
+            TableLoadOption::mysql { .. } => LoadedTable::new_from_df_table(Arc::new(
+                database::DatabaseLoader::MySQL.to_mem_table(t)?,
+            )),
+            TableLoadOption::sqlite { .. } => LoadedTable::new_from_df_table(Arc::new(
+                database::DatabaseLoader::SQLite.to_mem_table(t)?,
+            )),
+            TableLoadOption::postgres { .. } => LoadedTable::new_from_df_table(Arc::new(
+                database::DatabaseLoader::Postgres.to_mem_table(t)?,
+            )),
         })
     } else {
-        let t: Arc<dyn TableProvider> = match t.extension()? {
-            "csv" => csv::to_datafusion_table(t, dfctx).await?,
-            "json" => Arc::new(json::to_mem_table(t, dfctx).await?),
-            "ndjson" | "jsonl" => Arc::new(ndjson::to_mem_table(t, dfctx).await?),
-            "parquet" => parquet::to_datafusion_table(t, dfctx).await?,
-            "xls" | "xlsx" | "xlsb" | "ods" => Arc::new(excel::to_mem_table(t).await?),
-            "arrow" => Arc::new(arrow_ipc_file::to_mem_table(t, dfctx).await?),
-            "arrows" => Arc::new(arrow_ipc_stream::to_mem_table(t, dfctx).await?),
-            "mysql" => Arc::new(database::DatabaseLoader::MySQL.to_mem_table(t)?),
-            "sqlite" => Arc::new(database::DatabaseLoader::SQLite.to_mem_table(t)?),
-            "postgresql" => Arc::new(database::DatabaseLoader::Postgres.to_mem_table(t)?),
-            ext => {
-                return Err(Error::InvalidUri {
-                    msg: format!(
-                        "failed to register `{}` as table `{}`, unsupported table format `{}`",
-                        t.io_source, t.name, ext,
-                    ),
-                });
-            }
-        };
-
-        Ok(t)
+        match t.extension()? {
+            "csv" => csv::to_loaded_table(t.clone(), dfctx.clone()).await,
+            "json" => json::to_loaded_table(t.clone(), dfctx.clone()).await,
+            "ndjson" | "jsonl" => ndjson::to_loaded_table(t.clone(), dfctx.clone()).await,
+            "parquet" => parquet::to_loaded_table(t.clone(), dfctx.clone()).await,
+            "xls" | "xlsx" | "xlsb" | "ods" => excel::to_loaded_table(t.clone()).await,
+            "arrow" => arrow_ipc_file::to_loaded_table(t.clone(), dfctx.clone()).await,
+            "arrows" => arrow_ipc_stream::to_loaded_table(t.clone(), dfctx.clone()).await,
+            "mysql" => Ok(LoadedTable::new_from_df_table(Arc::new(
+                database::DatabaseLoader::MySQL.to_mem_table(t)?,
+            ))),
+            "sqlite" => Ok(LoadedTable::new_from_df_table(Arc::new(
+                database::DatabaseLoader::SQLite.to_mem_table(t)?,
+            ))),
+            "postgresql" => Ok(LoadedTable::new_from_df_table(Arc::new(
+                database::DatabaseLoader::Postgres.to_mem_table(t)?,
+            ))),
+            ext => Err(Error::InvalidUri {
+                msg: format!(
+                    "failed to register `{}` as table `{}`, unsupported table format `{}`",
+                    t.io_source, t.name, ext,
+                ),
+            }),
+        }
     }
 }
 
@@ -885,6 +978,18 @@ schema:
         );
     }
 
+    #[test]
+    fn test_table_name_from_path() {
+        assert_eq!(
+            table_name_from_path(
+                &URIReference::try_from("mysql://root:123456@1.1.1.1:3306/test")
+                    .unwrap()
+                    .path()
+            ),
+            Some("test".to_string()),
+        );
+    }
+
     #[cfg(feature = "database-sqlite")]
     #[tokio::test]
     async fn test_load_sqlite_table() {
@@ -892,8 +997,9 @@ schema:
 
         let t = TableSource::new("uk_cities", "sqlite://../test_data/sqlite/sample.db");
         let ctx = datafusion::prelude::SessionContext::new();
-        let table = load(&t, &ctx).await.unwrap();
-        let stats = table
+        let t = load(&t, &ctx).await.unwrap();
+        let stats = t
+            .table
             .scan(&ctx.state(), None, &[], None)
             .await
             .unwrap()
@@ -917,8 +1023,9 @@ uri: "sqlite://../test_data/sqlite/sample.{}"
             ))
             .unwrap();
             let ctx = datafusion::prelude::SessionContext::new();
-            let table = load(&t, &ctx).await.unwrap();
-            let stats = table
+            let t = load(&t, &ctx).await.unwrap();
+            let stats = t
+                .table
                 .scan(&ctx.state(), None, &[], None)
                 .await
                 .unwrap()
