@@ -5,9 +5,10 @@ use datafusion::arrow::datatypes::{Schema, SchemaRef};
 #[allow(deprecated)]
 use datafusion::arrow::json::reader::{infer_json_schema, ReaderBuilder};
 use datafusion::arrow::record_batch::RecordBatch;
+use datafusion::datasource::TableProvider;
 use snafu::prelude::*;
 
-use crate::table::{self, TableSource};
+use crate::table::{self, LoadedTable, TableSource};
 
 #[derive(Debug, Snafu)]
 pub enum Error {
@@ -89,6 +90,20 @@ pub async fn to_mem_table(
         .context(table::CreateMemTableSnafu)
 }
 
+async fn to_datafusion_table(
+    t: TableSource,
+    dfctx: datafusion::execution::context::SessionContext,
+) -> Result<Arc<dyn TableProvider>, table::Error> {
+    Ok(Arc::new(to_mem_table(&t, &dfctx).await?))
+}
+
+pub async fn to_loaded_table(
+    t: TableSource,
+    dfctx: datafusion::execution::context::SessionContext,
+) -> Result<LoadedTable, table::Error> {
+    LoadedTable::new_from_df_table_cb(move || to_datafusion_table(t.clone(), dfctx.clone())).await
+}
+
 #[cfg(test)]
 mod tests {
     use datafusion::{datasource::TableProvider, prelude::SessionContext};
