@@ -16,9 +16,9 @@ pub async fn schema<H: RoapiContext>(
 
 pub async fn get_by_table_name<H: RoapiContext>(
     state: extract::Extension<Arc<H>>,
-    extract::Path(table_name): extract::Path<&str>,
+    extract::Path(table_name): extract::Path<String>,
 ) -> Result<impl IntoResponse, ApiErrResp> {
     let ctx = state.0;
-    let payload = ctx.table_schema_json_bytes(table_name).await?;
+    let payload = ctx.table_schema_json_bytes(&table_name).await?;
     Ok(bytes_to_json_resp(payload))
 }
