@@ -19,37 +19,13 @@ pub enum Error {
     BuildHttpServer { source: server::http::Error },
     #[snafu(display("Failed to build FlightSQL server: {source}"))]
     BuildFlightSqlServer { source: server::flight_sql::Error },
-}
-
-pub struct TableReloader {
-    reload_interval: Duration,
-    ctx_ext: Arc<RwLock<RawRoapiContext>>,
-    tables: Arc<Mutex<HashMap<String, TableSource>>>,
-}
-
-impl TableReloader {
-    pub async fn run(self) {
-        let mut interval = time::interval(self.reload_interval);
-        loop {
-            interval.tick().await;
-            for (table_name, table) in self.tables.lock().await.iter() {
-                match self.ctx_ext.load_table(table).await {
-                    Ok(_) => {
-                        info!("table {} reloaded", table_name);
-                    }
-                    Err(err) => {
-                        error!("failed to reload table {}: {:?}", table_name, err);
-                    }
-                }
-            }
-        }
-    }
+    #[snafu(display("Failed to build context: {source}"))]
+    BuildContext { source: crate::context::Error },
 }
 
 pub struct Application {
     http_addr: std::net::SocketAddr,
-    http_server: server::http::HttpApiServer,
-    table_reloader: Option<TableReloader>,
+    http_server: server::http::HttpApiServe,
     postgres_server: Box<dyn server::RunnableServer>,
     flight_sql_server: Box<dyn server::RunnableServer>,
 }
@@ -58,9 +34,9 @@ impl Application {
     pub async fn build(config: Config) -> Result<Self, Error> {
         let default_host = std::env::var("HOST").unwrap_or_else(|_| "127.0.0.1".to_string());
 
-        let handler_ctx = RawRoapiContext::new(&config)
+        let handler_ctx = RawRoapiContext::new(&config, !config.disable_read_only)
             .await
-            .expect("Failed to create Roapi context");
+            .context(BuildContextSnafu)?;
 
         let tables = config
             .tables
@@ -70,7 +46,8 @@ impl Application {
         let tables = Arc::new(Mutex::new(tables));
 
         if config.disable_read_only {
-            let ctx_ext = Arc::new(RwLock::new(handler_ctx));
+            info!("Read-only mode disabled.");
+            let ctx_ext: Arc<ConcurrentRoapiContext> = Arc::new(RwLock::new(handler_ctx));
             let postgres_server = Box::new(
                 server::postgres::PostgresServer::new(
                     ctx_ext.clone(),
@@ -80,12 +57,6 @@ impl Application {
                 .await,
             );
 
-            let table_reloader = config.reload_interval.map(|reload_interval| TableReloader {
-                reload_interval,
-                tables: tables.clone(),
-                ctx_ext: ctx_ext.clone(),
-            });
-
             let flight_sql_server = Box::new(
                 server::flight_sql::RoapiFlightSqlServer::new(
                     ctx_ext.clone(),
@@ -97,23 +68,27 @@ impl Application {
             );
 
             let (http_server, http_addr) =
-                server::http::build_http_server::<ConcurrentRoapiContext>(
-                    ctx_ext,
-                    tables,
-                    &config,
-                    default_host,
-                )
-                .await
-                .context(BuildHttpServerSnafu)?;
+                server::http::build_http_server(ctx_ext.clone(), tables, &config, default_host)
+                    .await
+                    .context(BuildHttpServerSnafu)?;
+
+            let _handle = tokio::task::spawn(async move {
+                loop {
+                    if let Err(e) = ctx_ext.refresh_tables().await {
+                        error!("Failed to refresh table: {:?}", e);
+                    }
+                    time::sleep(Duration::from_millis(1000)).await;
+                }
+            });
 
             Ok(Self {
                 http_addr,
                 http_server,
                 postgres_server,
                 flight_sql_server,
-                table_reloader,
             })
         } else {
+            info!("Running in read-only mode.");
             let ctx_ext = Arc::new(handler_ctx);
             let postgres_server = Box::new(
                 server::postgres::PostgresServer::new(
@@ -146,7 +121,6 @@ impl Application {
                 http_server,
                 postgres_server,
                 flight_sql_server,
-                table_reloader: None,
             })
         }
     }
@@ -164,8 +138,6 @@ impl Application {
     }
 
     pub async fn run_until_stopped(self) -> Result<(), Error> {
-        // FIXME: exit from tokio spawn
-
         let postgres_server = self.postgres_server;
         info!(
             "🚀 Listening on {} for Postgres traffic...",
@@ -190,12 +162,6 @@ impl Application {
                 .expect("Failed to run FlightSQL server");
         });
 
-        if let Some(table_reloader) = self.table_reloader {
-            tokio::spawn(async move {
-                table_reloader.run().await;
-            });
-        }
-
         info!("🚀 Listening on {} for HTTP traffic...", self.http_addr);
         self.http_server.await.expect("Failed to start HTTP server");
 
